from ..source import *
'''
def test_multi_sectors():
    tic = 29987116 # for testing
    assert(len(multi_sectors('all', tic=tic)) is 1)
    assert(len(multi_sectors([1], tic=tic)) is 1)
    assert(len(multi_sectors([3,6,8], tic=tic)) is 0)
    try:
        multi_sectors(1, tic=tic)
    except TypeError:
        pass
    except Exception as e:
       self.fail('Unexpected exception raised:', e)
    else:
       self.fail('TypeError not raised')
'''
